#!/usr/bin/python
# -*- coding: utf-8 -*-
# ======================================================================
# 
# Project : freyja
# Author  : Terry 'Mongoose' Hendrix II
# Website : http://icculus.org/freyja/
# Email   : mongooseichiban@gmail.com
# Object  : 
# Comments: This is the binding generator for Lua.
#
# -- Notes ------------------------------------------------------------
#
# Enabling python support in libfreyja:
#
#  Set gForcePythonDefine = 1 for force compiliation of Python 
#  support, or add LUA_FOUND to compile flags in Makefile.
#
#  Then (re)build libfreyja.
#
# -- History ---------------------------------------------------------- 
#
# 2006.12.09:
# Mongoose: Created
# ======================================================================

import os
import re
import string
import time
import sys

gDateStamp = time.localtime( time.time() )
gForcePythonDefine = 0
gFuncWrappers = []
gFuncBindings = []

class ParseTree:
	_symbol = ""
	_parms = ""
	_children = []

	def __init__(self, symbol, parms):
		self._symbol = symbol
		self._parms = parms
		#self._children = children

	def AddChild(self, child):
		self._children.append( child )


# Yes, this is lame -- however the python file iterator has issues with
# multiline statement parsing on my machine even using f.next().
def GenerateParseTree( f ):
	buf = ""
	cache = False
	count = 0
	sym = False
	string = False

	cur = root = ParseTree("root", "")

	while True:
		c = f.read(1)
		buf += c

		if c == '(':
			cache = True
			tmp = ParseTree( "", "" )
			cur.AddChild( tmp )
			cur = tmp
			count = 0
		elif c == ')':
			cache = False
		elif c == '"':
			string = True
		elif string != True and c == ' ' or c == '\t' or c == '\n':
			nop = 0
		elif c == '':
			break;
		
		if cache:
			nop = 0

	# Strip comments //, newlines, and directives
	buf = re.sub('(//.*\n|\n|#.*\n)', '', buf)
	
	# Strip comments /* */
	buf = re.sub('\/\\*.*\\*\/', '', buf)

	# Strip out references
	buf = re.sub('&', '', buf)

	# Dump parsed 'lines' into generated file for debugging
	#print "/* $$$ " + buf + " $$$ */"

	return buf


def CountParms( s ):
	return s.count(',')


def ConvertParmToStringLiteral( s, a ):

	i = 0
	argA = s.partition(',')
	while i < a: 
		argA = argA[2].partition(',')
		i = i + 1

	s2 = re.sub(argA[0], "\"" + argA[0] + "\"", s)

	return s2


def SwapParms( s, a, b ):

	if s.count(',') == 0:
		return s

	i = 0
	argA = s.partition(',')
	while i < a: 
		argA = argA[2].partition(',')
		i = i + 1

	i = 0
	argB = s.partition(',')
	while i < b: 
		argB = argB[2].partition(',')
		i = i + 1

	#print argA[0]
	#print argB[0]

	s2 = re.sub(argA[0], 'OxXxXxXxO', s)
	s2 = re.sub(argB[0], argA[0], s2)
	s2 = re.sub('OxXxXxXxO', argB[0], s2)

	#print s2

	return s2


def RemoveParm( s, a ):

	if s.count(',') == 0:
		return s

	i = 0
	argA = s.partition(',')
	while i < a: 
		argA = argA[2].partition(',')
		i = i + 1

	if a > 0:
		s2 = re.sub(','+argA[0], '', s)
	else:
		s2 = re.sub(argA[0], '', s)

	return s2


def ConvertParmList( s ):

	l = len(s)
	j = 0
	p = 0

	s2 = ""

	while j < l:
		if s[j] == '"':
			p = not p

		if s[j] == ' ' and not p:
			s2 += ","
		else:
			s2 += s[j]

		j = j + 1

	return s2


def EndScope( s ):
	return  s.count('(') == s.count(')')


def ParseMlisp( buf ):

	lenght = len( buf )
	i = 0

	while True:
		line = ""

		while i < lenght:
			if buf[i] == '\n':
				break
			line += buf[i]
			i = i + 1

		if line == "":
			break

		#print line

		if line.find( '(submenu' ) == 0:
			s = re.sub('\t', ' ', line)
			s = re.sub('[ ]+', ' ', s)
			s = re.sub('\(submenu ', '', s)
			s = re.sub('[\)]+', '', s)
			s = ConvertParmList( s )
			s = SwapParms( s, 0, 1 )
			if CountParms( s ) > 0:
				s = ConvertParmToStringLiteral( s, 1 )
			s = re.sub(',', ', ', s)

			if EndScope( line ):
				print "\tmgtk_append_menu( submenu, mgtk_submenu( " + s + " ) )"
			else:
				print "\tsubmenu = mgtk_submenu( " + s + " )"	
				print "\tmgtk_append_menu( menubar, submenu )"
		elif line.find( '(menu_item' ) == 0:
			s = re.sub('\t', ' ', line)
			s = re.sub('[ ]+', ' ', s)
			s = re.sub('\(menu_item ', '', s)
			s = re.sub('[\)]+', '', s)
			s = ConvertParmList( s )
			s = ConvertParmToStringLiteral( s, 1 )
			s = re.sub(',', ', ', s)
			print "\tmgtk_append_menu( submenu, mgtk_menu_item( " + s + " ) )"
		elif line.find( '(menu_seperator' ) == 0:
			print "\tmgtk_append_menu( submenu, mgtk_menu_separator() )"		
		#(toggle_menu_item	"_Points"		eRenderVertex	0)	
		elif line.find( '(toggle_menu_item' ) == 0:
			s = re.sub('\t', ' ', line)
			s = re.sub('[ ]+', ' ', s)
			s = re.sub('\(toggle_menu_item ', '', s)
			s = re.sub('[\)]+', '', s)
			s = ConvertParmList( s )
			s = ConvertParmToStringLiteral( s, 1 )
			s = re.sub(',', ', ', s)
			print "\tmgtk_append_menu( submenu, mgtk_menu_item_check( " + s + " ) )"

		elif line.find( '(toolbar_button' ) == 0:
			s = re.sub('\t', ' ', line)
			s = re.sub('[ ]+', ' ', s)
			s = re.sub('\(toolbar_button ', '', s)
			s = re.sub('[\)]+', '', s)
			s = ConvertParmList( s )
			s = SwapParms( s, 1, 4 )
			s = SwapParms( s, 0, 2 )
			s = SwapParms( s, 0, 4 )
			#s = SwapParms( s, 1, 4 )
			#s = SwapParms( s, 3, 4 )
			#s = RemoveParm( s, 4 )
			s = ConvertParmToStringLiteral( s, 1 )
			s = re.sub(',', ', ', s)
			print "\tmgtk_toolbar_button(toolbar, " + s + " )"

		elif line.find( '(toolbar_togglebutton' ) == 0:
			s = re.sub('\t', ' ', line)
			s = re.sub('[ ]+', ' ', s)
			s = re.sub('\(toolbar_togglebutton ', '', s)
			s = re.sub('[\)]+', '', s)
			s = ConvertParmList( s )
			s = SwapParms( s, 0, 2 )
			s = SwapParms( s, 4, 5 )
			s = SwapParms( s, 1, 4 )
			s = SwapParms( s, 3, 4 )
			s = RemoveParm( s, 5 )
			s = ConvertParmToStringLiteral( s, 1 )
			s = re.sub(',', ', ', s)
			print "\tmgtk_toolbar_togglebutton(toolbar, " + s + " )"
		elif line.find( '(toolbar_separator' ) == 0:
			print "\tmgtk_toolbar_separator( toolbar )"	

	return True


def ImportMlisp( filename ):
	
	try:
		f = open( filename, "r" )

	except IOError:
		print "ERROR opening file '" + filename + "'" 
		return False

	# Preprocess the file to remove comments, etc.
	buf = ""
	tmp = ""

	while True:
		line = f.readline()

		if line == "":
			break
	
		# Strip comments, EOL, and leading/trailing whitespace.
		line = re.sub('(;.*|\n|^[\t ]*|[\t ]*$)', '', line)

		# Skip empty lines.
		if line == "":
			continue

		ParseMlisp( line )

		# FIXME: Handle ')' in strings.

#		tmp = tmp + line

		# Process lines with an ending of a scope.  
#		if tmp.find( ')' ):
#			sym = tmp
			#sym = re.sub('(\()', '<', sym)
			#sym = re.sub('(\))', '>', sym)
#			sym = re.sub('(\t)', ' ', sym)
#			buf = buf + sym + "\n"
#			tmp = ""
		
			#print sym #+ " />"	<-- if only 1 '(' found
			

	# For debugging dumps.
	#print buf

	#ParseMlisp( buf )

	f.close()

	return True



def ConvertMlispToLua( filename ):

	success = ImportMlisp( filename )

	#PrintOutFreyjaLuaABI()


if __name__ == "__main__":	
	args = sys.argv[1:]

	if not args:
		print "You must give a filename.mlisp as a parameter."
	else:
		ConvertMlispToLua(args[0])

