/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
/*===========================================================================
 * 
 * Project : Freyja
 * Author  : Terry 'Mongoose' Hendrix II
 * Website : http://icculus.org/freyja/
 * Email   : mongooseichiban@gmail.com
 * Object  : Camera
 * License : No use w/o permission (C) 2001-2006 Mongoose
 * Comments: Camera class for Freyja.
 *
 *           This file was generated using Mongoose's C++ 
 *           template generator script.  <mongoose@icculus.org>
 * 
 *-- History ------------------------------------------------ 
 *
 * 2006.08.03:
 * Mongoose - Created, reusing old libhel camera as a template
 ==========================================================================*/


#ifndef GUARD__FREYJA_MONGOOSE_CAMERA_H_
#define GUARD__FREYJA_MONGOOSE_CAMERA_H_

#include <hel/math.h>
#include <hel/Matrix.h>
#include <hel/Quaternion.h>
#include <hel/Vector.h>


namespace freyja3d {

class Camera
{
 public:

	////////////////////////////////////////////////////////////
	// Constructors
	////////////////////////////////////////////////////////////

	Camera();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Constructs an object of Camera
	 ------------------------------------------------------*/

	~Camera();
	/*------------------------------------------------------
	 * Pre  : Camera object is allocated
	 * Post : Deconstructs an object of Camera
	 ------------------------------------------------------*/


	////////////////////////////////////////////////////////////
	// Public Properties
	////////////////////////////////////////////////////////////

	const Vec3 &GetPosition() { return mPos; }
	void SetPosition(const Vec3 &pos) { mPos = pos; }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Current position
	 ------------------------------------------------------*/

	const Vec3 &GetSide()  { return mSide; }
	void SetSide(const Vec3 &side) { mSide = side; }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Side vector
	 ------------------------------------------------------*/

	const Vec3 &GetTarget() { return mTarget; }
	void SetTarget(const Vec3 &target) { mTarget = target; }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Returns target ( Look at position )
	 ------------------------------------------------------*/

	const Vec3 &GetUp()  { return mUp; }
	void SetUp(const Vec3 &up) { mUp = up; }
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Up vector
	 ------------------------------------------------------*/
	

	////////////////////////////////////////////////////////////
	// Public Accessors
	////////////////////////////////////////////////////////////
	
	bool IsAABBBehind(vec3_t min, vec3_t max);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Returns true if AABB is behind camera eye
	 ------------------------------------------------------*/

	bool IsPointBehind(vec3_t xyz);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Returns true if point is behind camera eye
	 ------------------------------------------------------*/


	////////////////////////////////////////////////////////////
	// Public Mutators
	////////////////////////////////////////////////////////////

	void Rotate(float angle, float x, float y, float z);
	/*------------------------------------------------------
	 * Pre  : <x>, <y>, <z> axis 
	 *        <angle> is in radians
	 * Post : Rotates camera
	 ------------------------------------------------------*/

	void Translate(Vec3 xyz);
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Moves the camera
	 ------------------------------------------------------*/

	void Update();
	/*------------------------------------------------------
	 * Pre  : 
	 * Post : Updates view target
	 ------------------------------------------------------*/


 private:

	////////////////////////////////////////////////////////////
	// Private Accessors
	////////////////////////////////////////////////////////////


	////////////////////////////////////////////////////////////
	// Private Mutators
	////////////////////////////////////////////////////////////

	Quaternion mOrn;            /* Quaternion for rotation */

	Vec3 mPos;                  /* Location in 3 space (aka eye) */
	
	Vec3 mTarget;               /* Postition we're looking at  */
	
	Vec3 mUp;                   /* Up vector  */
	
	Vec3 mSide;                 /* Side vector  */
			
	vec_t mDistance;            /* Distance from target */

	uint32 mFlags;              /* For testing with flags  */

	bool mUpdate;               /* Check to see if view needs updating */
};

} // namespace freyja3d

#endif // GUARD__FREYJA_MONGOOSE_CAMERA_H_
